/*
 * Copyright 2003 - 2013 The eFaps Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Revision:        $Rev: 8848 $
 * Last Changed:    $Date: 2013-02-19 10:49:59 -0500 (Tue, 19 Feb 2013) $
 * Last Changed By: $Author: jan@moxter.net $
 */

package org.efaps.admin.datamodel.attributetype;

import java.sql.SQLException;

import org.efaps.admin.datamodel.Attribute;
import org.efaps.db.wrapper.SQLInsert;
import org.efaps.db.wrapper.SQLUpdate;

/**
 * The class is the attribute type representation for the created date time of a
 * business object.
 *
 * @author The eFaps Team
 * @version $Id: CreatedType.java 8848 2013-02-19 15:49:59Z jan@moxter.net $
 */
public class CreatedType
    extends DateTimeType
{
    /**
     * The instance method appends
     * {@link org.efaps.db.databases.AbstractDatabase#getCurrentTimeStamp()} to
     * the SQL statement.
     *
     * @param _insert   insert statement
     * @param _attribute    attribute which is updated
     * @param _values   ignored, because always set to current time
     * @throws SQLException if SQL columns for the attribute are not correctly
     *                      defined
     */
    @Override
    public void prepareInsert(final SQLInsert _insert,
                              final Attribute _attribute,
                              final Object... _values)
        throws SQLException
    {
        checkSQLColumnSize(_attribute, 1);
        _insert.columnWithCurrentTimestamp(_attribute.getSqlColNames().get(0));
    }

    /**
     * An update of a type is not allowed and therefore a {@link SQLException}
     * is always thrown.
     *
     * @param _update       SQL update statement
     * @param _attribute    attribute which is updated
     * @param _values       new object value to set; values are localized and
     *                      are coming from the user interface
     * @throws SQLException allways, because not allowed
     */
    @Override
    public void prepareUpdate(final SQLUpdate _update,
                              final Attribute _attribute,
                              final Object... _values)
        throws SQLException
    {
        throw new SQLException("not allowed");
    }
}
