/*
 * Copyright 2003 - 2013 The eFaps Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Revision:        $Rev: 8896 $
 * Last Changed:    $Date: 2013-02-19 22:03:59 -0500 (Tue, 19 Feb 2013) $
 * Last Changed By: $Author: jan@moxter.net $
 */


package org.efaps.ci;

import org.efaps.util.cache.CacheReloadException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * Each class that extends this abstract class represents a configuration
 * item for a form from eFaps. It is used to have easy access during the
 * coding of esjp etc. to the configuration items without having the problem
 * of using strings to access them. The classes are created automatically
 * with a maven target.
 *
 * @author The eFaps Team
 * @version $Id: CIForm.java 8896 2013-02-20 03:03:59Z jan@moxter.net $
 */
//CHECKSTYLE:OFF
public abstract class CIForm
    extends CICollection
{
//CHECKSTYLE:ON
    /**
     * Logging instance used in this class.
     */
    private static final Logger LOG = LoggerFactory.getLogger(CIObject.class);

    /**
     * Constructor setting the uuid.
     * @param _uuid UUID of this type
     */
    protected CIForm(final String _uuid)
    {
        super(_uuid);
    }

    /**
     * Get the type this Configuration item represents.
     * @return Form
     */
    public org.efaps.admin.ui.Form getType()
    {
        org.efaps.admin.ui.Form ret = null;
        try {
            ret =  org.efaps.admin.ui.Form.get(this.uuid);
        } catch (final CacheReloadException e) {
            CIForm.LOG.error("Error on retrieving Type for CIType with uuid: {}", this.uuid);
        }
        return ret;
    }
}
